
define(['../../../models/request'], function(RequestModel) {

  describe('Request model', function() {

    var request;

    beforeEach(function() {
      request = new RequestModel({
        name: 'test_csr'
      });
    });

    it('expects the model ID to be the CSR name', function() {
      expect(request.id).toBe('test_csr');
    });

    it('defaults to pending status', function() {
      expect(request.get('status')).toBe('pending');
    });

    it('defaults to idle sync status', function() {
      expect(request.get('sync_status')).toBe('idle');
    });

    describe('setting its status', function() {
      it('should close the request and set its status to accepted when set to accepted', function() {
        request.set_accepted();

        expect(request.get('status')).toBe('accepted');
        expect(request.get('request_open')).toBeFalsy();
      });

      it('should close the request and set its status to rejected when set to accepted', function() {
        request.set_rejected();

        expect(request.get('status')).toBe('rejected');
        expect(request.get('request_open')).toBeFalsy();
      });

      it('should stay pending and set a message when set to failed', function() {
        request.set_failed('reason');

        expect(request.get('status')).toBe('pending');
        expect(request.get('message')).toBe('reason');
      })
    });

    describe('being signed', function() {

      var server;

      beforeEach(function() {
        server = sinon.fakeServer.create();
      });

      afterEach(function() {
        server.restore();
      });

      _.each(['accept', 'reject'], function(action) {

        var actioned = action + 'ed';

        describe('being ' + actioned, function() {

          it('should fire a sync event', function() {
            var callback = sinon.spy();

            request.on('sync:saving', callback);

            request[action]();

            expect(callback).toHaveBeenCalled();

          });

          it('should update its sync_status', function() {
            request[action]();

            expect(request.get('sync_status')).toBe('saving');
          });

          describe('when the server request succeeds', function() {

            beforeEach(function() {
              server.respondWith([200, {}, 'OK']);
            });

            it('should resolve a promise', function() {
              var callback = sinon.spy();

              request[action]().done(callback);

              server.respond();

              expect(callback).toHaveBeenCalled();
            });

            it('should update the request status', function() {
              request[action]();

              server.respond();

              expect(request.get('status')).toBe(actioned);
              expect(request.get('request_open')).toBeFalsy();
            });

            it('should fire a sync event', function() {
              var callback = sinon.spy();

              request.on('sync:complete', callback);

              request[action]();

              server.respond();

              expect(callback).toHaveBeenCalled();
            });

            it('should update its sync_status', function() {
              request[action]();

              server.respond();

              expect(request.get('sync_status')).toBe('complete');
            });

            it('should update the request status before firing the sync event', function() {
              var status,
                callback = function() {
                  status = request.get('status');
                };

              request.on('sync:complete', callback);

              request[action]();

              server.respond();

              expect(status).toBe(actioned);
            });
          });

          describe('when the server request fails', function() {

            beforeEach(function() {
              server.respondWith([403, {}, 'ERROR']);
            });

            it('should reject a promise with the error message', function() {
              var callback = sinon.spy();

              request[action]().fail(callback);

              server.respond();

              expect(callback).toHaveBeenCalledWith('ERROR');
            });

            it('should not update the request status', function() {
              request[action]();

              server.respond();

              expect(request.get('status')).toBe('pending');
            });

            it('should update the request with the reason for the error', function() {
              request[action]();

              server.respond();

              expect(request.get('message')).toBe('ERROR');
            });

            it('should fire a sync event', function() {
              var callback = sinon.spy();

              request.on('sync:failed', callback);

              request[action]();

              server.respond();

              expect(callback).toHaveBeenCalled();
            });

            it('should update its sync_status', function() {
              request[action]();

              server.respond();

              expect(request.get('sync_status')).toBe('failed');
            });

            it('should update the request with a reason for the error before firing a sync event', function() {
              var error,
                callback = function() {
                  error = request.get('message');
                };

              request.on('sync:failed', callback);

              request[action]();

              server.respond();

              expect(error).toBe('ERROR');
            });

          });
        });
      });
    });
  });

});